package org.herac.tuxguitar.player.impl.jsa.sequencer;

import javax.sound.midi.MidiUnavailableException;
import javax.sound.midi.Sequence;
import javax.sound.midi.Sequencer;
import javax.sound.midi.Transmitter;

import org.herac.tuxguitar.player.base.MidiPort;
import org.herac.tuxguitar.player.base.MidiPortEmpty;
import org.herac.tuxguitar.player.base.MidiSequenceHandler;
import org.herac.tuxguitar.player.base.MidiSequencer;

public class MidiSequencerImpl implements MidiSequencer,MidiSequenceLoader{

	private Sequencer sequencer;
	private MidiPort midiPort;

	public MidiSequencerImpl(Sequencer sequencer){
		this.sequencer = sequencer;
	}
	
	protected Sequencer getSequencer() throws MidiUnavailableException {    	
		try {
			if(!this.sequencer.isOpen()){
				this.sequencer.open();
				this.openReceiver();
			}
		} catch (Throwable throwable) {
			throw new MidiUnavailableException(throwable.getMessage());
		}
        return this.sequencer;
    }	

	public void open() {
		try {
			this.getSequencer();
		} catch (Throwable throwable) {
			throwable.printStackTrace();
		}		
	}	
	
	public void close() {
		try {
			if(this.sequencer != null && this.sequencer.isOpen()){       	
				this.sequencer.close();			
			}
		} catch (Throwable throwable) {
			throwable.printStackTrace();
		}			
	}

	 public void openReceiver(){
	   	try {
			Transmitter transmitter = getSequencer().getTransmitter();
	        transmitter.setReceiver(new MidiReceiverImpl(this));
		} catch (Throwable throwable) {
			throwable.printStackTrace();
		}   
	} 	

	public MidiSequenceHandler createSequence(int tracks) {
		return new MidiSequenceHandlerImpl(this,tracks);
	}

	public MidiPort getMidiPort() {
		if(this.midiPort == null){
			this.midiPort = new MidiPortEmpty();
		}
		return this.midiPort;
	}

	public void setMidiPort(MidiPort port) {
		this.midiPort = port;
	}
	
	public long getTickLength() {
		try {
			return getSequencer().getTickLength();
		} catch (Throwable throwable) {
			throwable.printStackTrace();
		}
		return 0;
	}

	public long getTickPosition() {
		try {
			return getSequencer().getTickPosition();
		} catch (Throwable throwable) {
			throwable.printStackTrace();
		}
		return 0;
	}


	public void setTickPosition(long tickPosition) {
		try {
			resetPicthBend();
			getSequencer().setTickPosition(tickPosition);
		} catch (Throwable throwable) {
			throwable.printStackTrace();
		}
	}
	
	public boolean isRunning() {
		try {
			return getSequencer().isRunning();
		} catch (Throwable throwable) {
			throwable.printStackTrace();
		}
		return false;
	}

	public void setMute(int index, boolean mute) {
		try {
			getSequencer().setTrackMute(index, mute);
		} catch (Throwable throwable) {
			throwable.printStackTrace();
		}
	}

	public void setSolo(int index, boolean solo) {
		try {
			getSequencer().setTrackSolo(index, solo);
		} catch (Throwable throwable) {
			throwable.printStackTrace();
		}
	}

	public void setSequence(Sequence sequence){
		try {
			getSequencer().setSequence(sequence);
		} catch (Throwable throwable) {
			throwable.printStackTrace();
		}
	}
	
	public void start() {
		try {				
			getSequencer().start();			
		} catch (Throwable throwable) {
			throwable.printStackTrace();
		}
	}

	public void stop() {
		try {
			this.getSequencer().stop();
			this.getMidiPort().out().sendAllNotesOff();
			this.getMidiPort().out().sendSystemReset();			
		} catch (Throwable throwable) {
			throwable.printStackTrace();
		}
	}

	public void resetPicthBend(){
		try {
			for(int channel = 0; channel < 16;channel ++){
				this.getMidiPort().out().sendPitchBend(channel, 64);
			}
		} catch (Throwable throwable) {
			throwable.printStackTrace();
		}		
	}

	public String getKey() {
		return this.sequencer.getDeviceInfo().getName();
	}

	public String getName() {
		return this.sequencer.getDeviceInfo().getName();
	}	
}
